package at.ac.sbg.zgis.configuration;

/**
 * Contains constants and fixed formats and repeatedly used strings that are used throughout the
 * program along with methods to produce them.
 */
public class Constants {
    public static final String separator = ",";
    public static final String dataContainerSeparator = ";";
    public static final String extDir = "eDiary";

    /**
     * Definition of SensorPlatforms
     */
    public enum SensorPlatform {
        UNDEFINED(0),
        PHONE(1),
        BIOHARNESS_3(2),
        EMPATICA_E4(3);

        private final int id;

        SensorPlatform(int id) {
            this.id = id;
        }

        public int getId() {
            return this.id;
        }
    }

    /**
     * Definition of Sensors. All sensors have a name, id and description, which will be exported
     * to the output database.
     */
    public enum Sensor {
        UNDEFINED(0),
        HEARTRATE(1),
        RESPIRATIONRATE(2),
        SKINTEMPERATURE(3),
        POSTURE(4),
        PEAKACCELERATION(5),
        ROGSTATUS(6, "ROG: Fitness index - Red, Orange, Green"),
        GSR(7, "GSR: Galvanic skin response"),
        BREATHINGWAVEAMPLITUDE(8),
        ECGAMPLITUDE(9),
        ECGNOISE(10),
        XYZACCELERATIONMINMAX(11),
        VMU(12, "VMU: Vector magnitude units - Activity measure"),
        WORNSTATUS(13, "Worn Status: shows if the device is worn"),
        BREATHING(14),
        ECG(15, "ECG: Electrocardiogram"),
        RRDATA(16, "RR: intervals between two R-peaks in QRS complex"),
        ACCELERATIONDATA(17),
        GYROSCOPEDATA(18),
        SPEED(19),
        PEDOMETER(20),
        BVP(21, "BVP: Blood volume pressure"),
        IBI(22, "IBI: Inter beat interval");

        private final int id;
        private final String description;

        Sensor(int id, String description) {
            this.id = id;
            this.description = description;
        }

        Sensor(int id) {
            this.id = id;
            this.description = "";
        }

        public int getId() {
            return this.id;
        }

        public String getDescription() {
            return this.description;
        }
    }
}
