package at.ac.sbg.zgis.location;

import android.Manifest;
import android.app.Notification;
import android.app.NotificationChannel;
import android.app.NotificationManager;
import android.app.Service;
import android.content.Context;
import android.content.Intent;
import android.content.pm.PackageManager;
import android.location.Location;
import android.location.LocationListener;
import android.location.LocationManager;
import android.os.Binder;
import android.os.Bundle;
import android.os.IBinder;
import android.support.v4.app.ActivityCompat;
import android.util.Log;

import at.ac.sbg.zgis.recording.Recording;
import at.ac.sbg.zgis.userinterface.R;

public class eDiaryLocationService extends Service implements LocationListener {

    private static final int NOTIFICATION_ID = 1;
    private final String TAG = eDiaryLocationService.class.getSimpleName();

    private final IBinder binder = new LocalBinder();

    public class LocalBinder extends Binder {
        // TODO: 15.01.20 Jakob: Android Studio says this function can be void (?)
        public eDiaryLocationService getService() {
            return eDiaryLocationService.this;
        }
    }

    @Override
    public IBinder onBind(Intent intent) {
        Log.d(TAG, "on Bind()");
        return binder;
    }


    @Override
    public int onStartCommand(Intent intent, int flags, int startId) {
        final String channelId = "channelId_1234";
        NotificationManager notificationManager = getSystemService(NotificationManager.class);
        if (notificationManager != null) {
            final NotificationChannel channel;
            if (android.os.Build.VERSION.SDK_INT >= android.os.Build.VERSION_CODES.O) {

                channel = new NotificationChannel(channelId,
                        getResources().getString(R.string.app_name),
                        NotificationManager.IMPORTANCE_DEFAULT);

                notificationManager.createNotificationChannel(channel);
                Notification.Builder b = new Notification.Builder(this, channelId);

                b.setSmallIcon(R.mipmap.icon)
                        .setContentTitle(getResources().getString(R.string.app_name))
                        .setContentText(getResources().getString(R.string.application_is_running));
                startForeground(NOTIFICATION_ID, b.build());
            }
        }

        return super.onStartCommand(intent, flags, startId);
    }

    @Override
    public void onCreate() {
        Log.d(TAG, "onCreate()");

        LocationManager locationManager = (LocationManager) this.getSystemService(Context.LOCATION_SERVICE);

        if (ActivityCompat.checkSelfPermission(this, Manifest.permission.ACCESS_FINE_LOCATION) != PackageManager.PERMISSION_GRANTED && ActivityCompat.checkSelfPermission(this, Manifest.permission.ACCESS_COARSE_LOCATION) != PackageManager.PERMISSION_GRANTED) {
            return;
        }
        if (locationManager != null) {
            locationManager.requestLocationUpdates(LocationManager.GPS_PROVIDER, 1000, 0, this);

            // activate Network only if it is available
            if (locationManager.isProviderEnabled(LocationManager.NETWORK_PROVIDER)) {
                locationManager.requestLocationUpdates(LocationManager.NETWORK_PROVIDER, 1000, 0, this);
            }
        }
    }

    @Override
    public void onDestroy() {
        Log.d(TAG, "onDestroy()");
    }

    @Override
    public void onLocationChanged(Location location) {
        Log.i(TAG, "onLocationChanged() - location: " + location.toString());

        Recording recording = Recording.getInstance();
        recording.setLastLocation(location);
        recording.submitLocationToDB(location);

    }

    @Override
    public void onStatusChanged(String provider, int status, Bundle extras) {
        Log.d(TAG, "onStatusChanged()");
    }

    @Override
    public void onProviderEnabled(String provider) {
        Log.d(TAG, "onProviderEnabled()");
    }

    @Override
    public void onProviderDisabled(String provider) {
        Log.d(TAG, "onProviderDisabled()");
    }
}
