package at.ac.sbg.zgis.sensors;

import android.app.IntentService;
import android.content.Intent;
import android.util.Log;

import at.ac.sbg.zgis.configuration.Constants;

/**
 * Used to define and manage sensor platforms and packet types. This includes not only the types of
 * measurement that are carried out by each sensor, but also the connectivity state and battery status.
 */
public abstract class SensorPlatform extends IntentService {

    private static final String LOG_TAG = "SensorPlatform";

    private final Constants.SensorPlatform sensorPlatform;
    private ConnectionStatus connectionStatus = ConnectionStatus.DISCONNECTED;
    private float batteryStatus = 0f;

    /**
     * Creates an IntentService.
     *
     * @param platform The SensorPlatform the respective subclass is representing.
     */
    public SensorPlatform(Constants.SensorPlatform platform) {
        //The name of the SensorPlatform is provided to the IntentService constructor for debugging purposes.
        super(platform.name());
        this.sensorPlatform = platform;
    }

    protected void setConnectionStatus(ConnectionStatus connectionStatus) {
        this.connectionStatus = connectionStatus;
        Log.d(LOG_TAG, "Connection status set: " + this.getSensorPlatformName() + " + " + this.connectionStatus);
        SensorStatusCollection.update(this.getSensorPlatform(), this.connectionStatus, String.valueOf(this.batteryStatus));
    }

    protected void setBatteryStatus(float batteryStatus) {
        this.batteryStatus = batteryStatus;
        SensorStatusCollection.update(this.getSensorPlatform(), this.connectionStatus, String.valueOf(this.batteryStatus));
    }

    private String getSensorPlatformName() {
        return this.getSensorPlatform().name();
    }

    private Constants.SensorPlatform getSensorPlatform() {
        return this.sensorPlatform;
    }

    @Override
    abstract protected void onHandleIntent(Intent intent);

    /**
     * Every sensorPlatform has a ConnectionStatus, which is updated in the respective sensorPlatform class
     * and displayed in the sensorPlatform status GUI.
     */
    public enum ConnectionStatus {
        CONNECTED,
        CONNECTING,
        DISCONNECTED,
        DISCOVERING,
    }
}