package at.ac.sbg.zgis.sensors.empatica;

import android.bluetooth.BluetoothAdapter;
import android.content.Intent;
import android.os.AsyncTask;
import android.util.Log;

import com.empatica.empalink.ConnectionNotAllowedException;
import com.empatica.empalink.EmpaDeviceManager;
import com.empatica.empalink.EmpaticaDevice;
import com.empatica.empalink.config.EmpaSensorType;
import com.empatica.empalink.config.EmpaStatus;
import com.empatica.empalink.delegate.EmpaDataDelegate;
import com.empatica.empalink.delegate.EmpaStatusDelegate;

import at.ac.sbg.zgis.configuration.Constants;
import at.ac.sbg.zgis.sensors.SensorDataBus;
import at.ac.sbg.zgis.sensors.SensorDataCollection;
import at.ac.sbg.zgis.sensors.SensorDataEntry;
import at.ac.sbg.zgis.sensors.SensorPlatform;

/**
 * This class handles the Empatica E4 wrist band sensor.
 */
public class E4 extends SensorPlatform {

    private static final String TAG = E4.class.getSimpleName();
    private static final Constants.SensorPlatform SENSOR_PLATFORM = Constants.SensorPlatform.EMPATICA_E4;
    public static String EMPATICA_API_KEY = "";

    /**
     * Creates an IntentService.  Invoked by your subclass's constructor.
     */
    public E4() {
        super(SENSOR_PLATFORM);
    }

    // entry point of the E4 plugin. It is called from SensorPlugin.java
    @Override
    protected void onHandleIntent(Intent intent) {

        //Update connection status for GUI
        setConnectionStatus(ConnectionStatus.DISCONNECTED);

        Log.d(TAG, "Empatica E4 intent NEW: " + intent);
        appTask task = new appTask();
        task.doInBackground();

    }


    private class appTask extends AsyncTask<Void, Void, Void> implements EmpaDataDelegate, EmpaStatusDelegate {
        private EmpaDeviceManager empaDeviceManager = null;

        /**
         * Initialize empaDeviceManager and authenticate with the API key.
         * If the device is turned on, it will try to connect after this.
         */
        @Override
        protected Void doInBackground(Void... params) {
            //Update connection status for GUI
            setConnectionStatus(ConnectionStatus.CONNECTING);
            this.empaDeviceManager = new EmpaDeviceManager(getApplicationContext(), this, this);
            this.empaDeviceManager.authenticateWithAPIKey(EMPATICA_API_KEY);
            return null;
        }


        @Override
        //GSR = galvanic skin response
        public void didReceiveGSR(float gsr, double timestamp) {
            SensorDataCollection sensorDataCollection = new SensorDataCollection(E4.SENSOR_PLATFORM);

            SensorDataEntry sensorDataEntry = new SensorDataEntry(Constants.Sensor.GSR, gsr);
            sensorDataCollection.addDataEntry(sensorDataEntry);
            SensorDataBus.dataAvailable(sensorDataCollection);
        }

        @Override
        //BVP = blood volume pressure
        public void didReceiveBVP(float bvp, double timestamp) {
            SensorDataCollection sensorDataCollection = new SensorDataCollection(E4.SENSOR_PLATFORM);

            SensorDataEntry sensorDataEntry = new SensorDataEntry(Constants.Sensor.BVP, bvp);
            sensorDataCollection.addDataEntry(sensorDataEntry);
            SensorDataBus.dataAvailable(sensorDataCollection);
        }

        @Override
        //IBI = interbeat interval
        public void didReceiveIBI(float ibi, double timestamp) {
            SensorDataCollection sensorDataCollection = new SensorDataCollection(E4.SENSOR_PLATFORM);

            SensorDataEntry sensorDataEntry = new SensorDataEntry(Constants.Sensor.IBI, ibi);
            sensorDataCollection.addDataEntry(sensorDataEntry);
            SensorDataBus.dataAvailable(sensorDataCollection);
        }

        @Override
        public void didReceiveTemperature(float temperature, double timestamp) {
            SensorDataCollection sensorDataCollection = new SensorDataCollection(E4.SENSOR_PLATFORM);

            SensorDataEntry sensorDataEntry = new SensorDataEntry(Constants.Sensor.SKINTEMPERATURE, temperature);
            sensorDataCollection.addDataEntry(sensorDataEntry);
            SensorDataBus.dataAvailable(sensorDataCollection);
        }

        @Override
        public void didReceiveAcceleration(int accX, int accY, int accZ, double timestamp) {
            SensorDataCollection sensorDataCollection = new SensorDataCollection(E4.SENSOR_PLATFORM);

            String dataString = accX + Constants.separator;
            dataString += accY + Constants.separator;
            dataString += accZ;

            SensorDataEntry sensorDataEntry = new SensorDataEntry(Constants.Sensor.ACCELERATIONDATA, dataString);
            sensorDataCollection.addDataEntry(sensorDataEntry);
            SensorDataBus.dataAvailable(sensorDataCollection);
        }

        @Override
        public void didReceiveBatteryLevel(float batteryLevel, double timestamp) {

            setBatteryStatus(batteryLevel * 100);
        }

        @Override
        public void didReceiveTag(double timestamp) {
        }

        /**
         * Translate E4 status message to "app"-status messages
         * like defined in SensorStatus.ConnectionStatus
         */
        @Override
        public void didUpdateStatus(EmpaStatus status) {
            Log.d(TAG, "E4 Status: " + status);

            // translate from "E4" status and "App connection" status
            // in order to be better understandable for the user
            switch (status) {
                case CONNECTED:
                    setConnectionStatus(ConnectionStatus.CONNECTED);

                    break;
                case READY:
                    setConnectionStatus(ConnectionStatus.DISCONNECTED);
                    empaDeviceManager.startScanning();

                    break;
                case CONNECTING:
                    setConnectionStatus(ConnectionStatus.CONNECTING);

                    break;
                default:
                    setConnectionStatus(ConnectionStatus.DISCONNECTED);
                    break;
            }
        }

        @Override
        public void didEstablishConnection() {

        }

        @Override
        public void didUpdateSensorStatus(int status, EmpaSensorType type) {

        }


        @Override
        public void didDiscoverDevice(EmpaticaDevice device, String deviceLabel, int rssi, boolean allowed) {
            if (allowed) {
                empaDeviceManager.stopScanning();
                try {
                    //connect to device
                    empaDeviceManager.connectDevice(device);
                } catch (ConnectionNotAllowedException ex) {
                    System.err.println(ex.getMessage());
                    Log.e(TAG, "didDiscoverDevice()", ex);
                }
            }
        }

        @Override
        public void didRequestEnableBluetooth() {
            Intent enableBtIntent = new Intent(BluetoothAdapter.ACTION_REQUEST_ENABLE);
            startService(enableBtIntent);
        }

        @Override
        public void didUpdateOnWristStatus(int status) {

        }

    }

}
