package at.ac.sbg.zgis.survey;

import android.location.Location;
import android.util.SparseArray;

import java.util.Calendar;



/**
 * Contains survey items, location and time
 * serves both as storage of answers and as blueprint for creating DB table and UI
 */
public class Survey {

    private Location location = null;
    private Calendar timestamp = null;

    // Android Studio suggest to use a SparseArray instead of a HashMap
    private final SparseArray<SurveyItem> surveyItems;

    public Survey(SparseArray<SurveyItem> surveyItemsSparseArray) {
        this.surveyItems = surveyItemsSparseArray;
    }

    public Location getLocation() {
        return location;
    }

    public void setLocation(Location location) {
        this.location = location;
    }

    public Calendar getTimestamp() {
        return timestamp;
    }

    public void setTimestamp(Calendar timestamp) {
        this.timestamp = timestamp;
    }

    public SparseArray<SurveyItem> getSurveyItems() {
        return surveyItems;
    }

    /**
     * Resets all survey answers, the location and the timestamp
     */
    public void resetSurvey() {

        for (int i = 0; i < this.surveyItems.size(); i++) {
            SurveyItem surveyItem = this.surveyItems.get(i);
            surveyItem.makeAnswerEmpty();
            this.surveyItems.put(i, surveyItem);
        }

        this.location = null;
        this.timestamp = null;
    }

    @Override
    public String toString() {
        return "Survey{" +
                "location=" + location +
                ", timestamp=" + timestamp +
                ", surveyItems=" + surveyItems +
                '}';
    }
}
