package at.ac.sbg.zgis.survey;

import java.io.Serializable;

/**
 * Parent class of the survey items like Slider or Multiple Choice
 *
 * @param <AnswerGenericType> String or int
 */
public abstract class SurveyItem<AnswerGenericType> implements Serializable {

    protected final String questionOriginal;

    // the DB columns directly contain the question (removed whitespace, special characters, etc.)
    // this rather pragmatic approach is chosen, because the alternative would be to create a elaborate
    // DB schema which would cause more complexity in the code base
    protected final String questionDBColumnName;

    protected AnswerGenericType userAnswer = null;

    protected SurveyItem(String question) {
        this.questionOriginal = question;
        this.questionDBColumnName = questionStringToColumnName(question);
    }

    /**
     * Removes whitespace and special characters
     */
    private static String questionStringToColumnName(String question) {
        return question.
                trim(). // remove trailing white space
                toLowerCase().
                replace(" ", "_"). // whitespace to underscore
                replace(":", "_"). // colon to underscore
                replace("?", ""); // remove question mark
    }

    public String getQuestion() {
        return questionOriginal;
    }

    public void submitAnswer(AnswerGenericType userAnswer) {
        this.userAnswer = userAnswer;
    }

    public boolean isAnswered() {
        return (this.userAnswer != null);
    }

    public AnswerGenericType getUserAnswer() {
        return userAnswer;
    }

    public void makeAnswerEmpty() {
        this.userAnswer = null;
    }

    // TODO: 08.05.18 handle special characters like "äüößÜÖÄ"

    public String getQuestionDBColumnName() {
        return questionDBColumnName;
    }
}
