package at.ac.sbg.zgis.userinterface;

import android.os.Bundle;
import android.support.v4.app.Fragment;
import android.support.v4.app.FragmentActivity;
import android.support.v4.app.FragmentManager;
import android.support.v4.app.FragmentPagerAdapter;
import android.support.v4.view.PagerAdapter;
import android.support.v4.view.ViewPager;
import android.util.SparseArray;

import java.util.ArrayList;

import at.ac.sbg.zgis.configuration.Configuration;
import at.ac.sbg.zgis.recording.Recording;
import at.ac.sbg.zgis.survey.Survey;
import at.ac.sbg.zgis.survey.SurveyItem;

/**
 * Contains the ViewPager for the survey questions
 */
public class SurveyActivity extends FragmentActivity {

    public static final String SURVEY_ITEM_INDEX = "survey_item_index";
    public static final String SURVEY_SIZE = "survey_size";
    private ViewPager viewPager;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_survey);

        // get empty survey
        Recording recording = Recording.getInstance();

        // get empty survey
        recording.resetSurvey();
        Survey survey = recording.getSurvey();

        // fill fragments
        final ArrayList<Fragment> fragments = new ArrayList<>();

        SparseArray<SurveyItem> surveyItems = survey.getSurveyItems();

        int surveySize = surveyItems.size();
        for (int i = 0; i < surveySize; i++) {

            Fragment fragment = new SurveyFragment();
            Bundle bundle = new Bundle();

            SurveyItem surveyItem = surveyItems.get(i);
            bundle.putInt(SURVEY_ITEM_INDEX, i);
            bundle.putInt(SURVEY_SIZE, surveySize);
            bundle.putSerializable(Configuration.QUESTION, surveyItem);

            fragment.setArguments(bundle);
            fragments.add(fragment);
        }

        // Instantiate a ViewPager and a PagerAdapter.
        viewPager = findViewById(R.id.vp_survey_view_pager);
        PagerAdapter pagerAdapter = new SurveyPagerAdapter(getSupportFragmentManager(), fragments);
        viewPager.setAdapter(pagerAdapter);
    }

    /**
     * Only called by the children fragments
     */
    public void goToNextPage() {
        viewPager.setCurrentItem(viewPager.getCurrentItem() + 1);
    }

    @Override
    public void onBackPressed() {
        if (viewPager.getCurrentItem() == 0) {
            // if first fragment, return to main activity
            super.onBackPressed();
        } else {
            // otherwise go to previous fragment
            viewPager.setCurrentItem(viewPager.getCurrentItem() - 1);
        }
    }

    private class SurveyPagerAdapter extends FragmentPagerAdapter {
        private final ArrayList<Fragment> fragments;

        SurveyPagerAdapter(FragmentManager fm, ArrayList<Fragment> fragments) {
            super(fm);
            this.fragments = fragments;
        }

        @Override
        public Fragment getItem(int position) {
            return this.fragments.get(position);
        }

        @Override
        public int getCount() {
            return this.fragments.size();
        }
    }
}
