package at.ac.sbg.zgis.userinterface;

import android.app.AlertDialog;
import android.content.DialogInterface;
import android.os.Bundle;
import android.support.v4.app.Fragment;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.Button;
import android.widget.EditText;
import android.widget.FrameLayout;
import android.widget.RadioButton;
import android.widget.RadioGroup;
import android.widget.SeekBar;
import android.widget.TextView;
import android.widget.Toast;

import java.util.ArrayList;

import at.ac.sbg.zgis.configuration.Configuration;
import at.ac.sbg.zgis.recording.Recording;
import at.ac.sbg.zgis.survey.SurveyItem;
import at.ac.sbg.zgis.survey.surveyitem.RadioButtonGroup;
import at.ac.sbg.zgis.survey.surveyitem.Slider;
import at.ac.sbg.zgis.survey.surveyitem.TextInput;

public class SurveyFragment extends Fragment {

    private SurveyItem surveyItem;
    private Recording recording;
    private int surveyItemIndex;

    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container,
                             Bundle savedInstanceState) {
        ViewGroup rootView = (ViewGroup) inflater.inflate(R.layout.fragment_survey, container, false);

        // access Recording singleton
        recording = Recording.getInstance();

        // data from parent fragment
        surveyItem = (SurveyItem) getArguments().getSerializable(Configuration.QUESTION);
        surveyItemIndex = getArguments().getInt(SurveyActivity.SURVEY_ITEM_INDEX);
        int surveySize = getArguments().getInt(SurveyActivity.SURVEY_SIZE);


        // question number
        TextView txtQuestionNr = rootView.findViewById(R.id.txt_question_number);
        int questionNr = surveyItemIndex + 1;
        txtQuestionNr.setText(getResources().getString(R.string.question_number, questionNr, surveySize));


        // question
        TextView txtQuestion = rootView.findViewById(R.id.txt_question);
        String question = surveyItem.getQuestion();
        txtQuestion.setText(question);


        // Button Cancel
        Button btnCancel = rootView.findViewById(R.id.btn_cancel);
        btnCancel.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                new AlertDialog.Builder(getContext())
                        .setTitle(getResources().getString(R.string.cancel_survey))
                        .setMessage(getResources().getString(R.string.question_cancel_survey))
                        .setNegativeButton(getResources().getString(R.string.cancel_survey),
                                new DialogInterface.OnClickListener() {
                                    public void onClick(DialogInterface dialog, int id) {
                                        getActivity().finish();
                                    }
                                })
                        .setPositiveButton(getResources().getString(R.string.continue_string), new DialogInterface.OnClickListener() {
                            public void onClick(DialogInterface dialog, int id) {
                                recording.getSurvey().resetSurvey();
                                dialog.cancel();
                            }
                        }).create().show();
            }
        });


        // Button Back
        Button btnBack = rootView.findViewById(R.id.btn_back);
        btnBack.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                getActivity().onBackPressed();
            }
        });


        // User Input
        final View surveyInputView;
        if (surveyItem instanceof RadioButtonGroup) {

            surveyInputView = new RadioGroup(getContext());
            ((RadioGroup) surveyInputView).setOrientation(RadioGroup.VERTICAL);

            // add answers
            ArrayList<String> answerOptions = ((RadioButtonGroup) surveyItem).getAnswerOptions();
            for (int i = 0; i < answerOptions.size(); i++) {
                String answerValue = answerOptions.get(i);
                RadioButton radioButton = new RadioButton(getContext());
                radioButton.setText(answerValue);
                radioButton.setId(i);
                ((RadioGroup) surveyInputView).addView(radioButton);
            }

            // set already chosen value ( in case user goes back)
            RadioButtonGroup existingItem = (RadioButtonGroup) recording.getSurvey().getSurveyItems().get(surveyItemIndex);

            if (existingItem.isAnswered()) {
                String existingAnswer = existingItem.getUserAnswer();

                int position = answerOptions.indexOf(existingAnswer);
                ((RadioGroup) surveyInputView).check(position);
            }


        } else if (surveyItem instanceof Slider) {
            // Seekbar a.k.a. Slider
            surveyInputView = new SeekBar(getContext());
            int max = ((Slider) surveyItem).getMaximum();
            ((SeekBar) surveyInputView).setMax(max);

            // set already chosen value ( in case user goes back)
            Slider existingItem = (Slider) recording.getSurvey().getSurveyItems().get(surveyItemIndex);

            if (existingItem.isAnswered()) {
                int existingAnswer = existingItem.getUserAnswer();
                ((SeekBar) surveyInputView).setProgress(existingAnswer);
            }
        } else {
            // TextInput
            surveyInputView = new EditText(getContext());

            // set already chosen value ( in case user goes back)
            TextInput existingItem = (TextInput) recording.getSurvey().getSurveyItems().get(surveyItemIndex);

            if (existingItem.isAnswered()) {
                String existingAnswer = existingItem.getUserAnswer();
                ((EditText) surveyInputView).setText(existingAnswer);
            }
        }

        // add respective user input element to UI
        FrameLayout frameLayout = rootView.findViewById(R.id.fl_survey_input);
        frameLayout.removeAllViews();
        FrameLayout.LayoutParams params = new FrameLayout.LayoutParams(FrameLayout.LayoutParams.MATCH_PARENT, FrameLayout.LayoutParams.WRAP_CONTENT);
        frameLayout.addView(surveyInputView, params);
        frameLayout.forceLayout();


        // Button "Next"
        Button btnNext = rootView.findViewById(R.id.btn_next);

        // Change button text
        final boolean lastQuestion = ((surveyItemIndex + 1) == surveySize);
        if (lastQuestion) {
            btnNext.setText(getResources().getString(R.string.submit_survey));
        }

        btnNext.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {

                // enters user input if valid
                boolean userInputIsValid = false;

                if (surveyInputView instanceof RadioGroup) {
                    int answerIndex = ((RadioGroup) surveyInputView).getCheckedRadioButtonId();
                    // -1 means nothing is checked
                    if (answerIndex >= 0) {
                        String selectedItem = ((RadioButtonGroup) surveyItem).getAnswerOptions().get(answerIndex);
                        userInputIsValid = true;
                        // submit user input
                        surveyItem.submitAnswer(selectedItem);
                    }

                } else if (surveyInputView instanceof SeekBar) {
                    int seekbarValue = ((SeekBar) surveyInputView).getProgress();
                    if (seekbarValue > 0) {
                        userInputIsValid = true;
                        // submit user input
                        surveyItem.submitAnswer(seekbarValue);
                    }

                } else {
                    // TODO: 23.08.18 handle case properly
                    String selectedItem = ((EditText) surveyInputView).getText().toString();
                    if (!selectedItem.isEmpty()) {
                        userInputIsValid = true;
                        // submit user input
                        surveyItem.submitAnswer(selectedItem);
                    }else {
                        userInputIsValid = true;
                    }
                }

                if (userInputIsValid) {
                    recording.enterAnsweredSurveyItem(surveyItemIndex, surveyItem);

                    if (lastQuestion) {
                        // submit to DB
                        if (recording.getLastLocation() != null){
                            recording.getSurvey().setTimestamp(eDiary.getTimeStamp());
                            recording.getSurvey().setLocation(recording.getLastLocation());

                            recording.submitSurveyToDB();
                            Toast.makeText(getContext(), getResources().getString(R.string.survey_has_been_submitted), Toast.LENGTH_LONG).show();

                            // returns to main screen without restart of app
                            getActivity().finish();
                        }

                        else {
                            Toast.makeText(getContext(), getResources().getString(R.string.location_unavailable), Toast.LENGTH_LONG).show();
                        }

                    } else {
                        // go to next page
                        ((SurveyActivity) getActivity()).goToNextPage();
                    }
                } else {
                    // user input not valid
                    Toast.makeText(getContext(), getResources().getString(R.string.please_answer_all_questions), Toast.LENGTH_LONG).show();
                }
            }
        });

        return rootView;
    }
}