package at.ac.sbg.zgis.userinterface;

import android.Manifest;
import android.annotation.SuppressLint;
import android.app.AlarmManager;
import android.app.AlertDialog;
import android.app.AlertDialog.Builder;
import android.app.PendingIntent;
import android.bluetooth.BluetoothAdapter;
import android.bluetooth.BluetoothDevice;
import android.content.ActivityNotFoundException;
import android.content.BroadcastReceiver;
import android.content.ComponentName;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.IntentFilter;
import android.content.ServiceConnection;
import android.content.pm.PackageManager;
import android.location.Location;
import android.location.LocationManager;
import android.net.ConnectivityManager;
import android.net.NetworkInfo;
import android.net.Uri;
import android.os.Build;
import android.os.Bundle;
import android.os.Handler;
import android.os.IBinder;
import android.os.PowerManager;
import android.os.Process;
import android.provider.Settings;
import android.support.v4.app.ActivityCompat;
import android.support.v4.content.ContextCompat;
import android.support.v7.app.AppCompatActivity;
import android.util.Log;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.widget.Button;
import android.widget.TextView;
import android.widget.Toast;

import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Calendar;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Objects;
import java.util.TimeZone;

import at.ac.sbg.zgis.configuration.Configuration;
import at.ac.sbg.zgis.configuration.Constants;
import at.ac.sbg.zgis.location.eDiaryLocationService;
import at.ac.sbg.zgis.marker.Marker;
import at.ac.sbg.zgis.recording.Recording;
import at.ac.sbg.zgis.sensors.SensorManager;
import at.ac.sbg.zgis.sensors.SensorPlatform;
import at.ac.sbg.zgis.sensors.SensorStatusCollection;
import at.ac.sbg.zgis.sensors.SensorStatusEntry;
import at.ac.sbg.zgis.sensors.zephyr.BioHarness3;
import at.ac.sbg.zgis.survey.Survey;
import at.ac.sbg.zgis.util.Utils;


/**
 * Main class of the app
 */
public class eDiary extends AppCompatActivity {

    // TimeStamp format for ISO String
    public static final String timeFormatISOstring = "yyyy-MM-dd'T'HH:mm:ss.SSS";

    // The SimpleDateFormat used in this class (SDF is not thread-safe!)
    @SuppressLint("SimpleDateFormat")

    private static final String TAG = eDiary.class.getSimpleName();

    //Bluetooth request variable needed for the dialog
    private static final int REQUEST_ENABLE_BT = 1;

    // This is the context which is used by other classes
    public static Context context = null;

    private final SensorManager sensorManager = new SensorManager();

    // TODO: 15.03.2018 some dangerous permissions are missing in this array
    // Array with "dangerous" permissions required to run app.
    private final String[] permissions = {
            Manifest.permission.ACCESS_FINE_LOCATION,
            Manifest.permission.WRITE_EXTERNAL_STORAGE
    };

    private PowerManager.WakeLock wakeLock;

    // Sensor Status Info
    private TextView txtE4ConnectionStatus;
    private TextView txtE4BatteryStatus;
    private Handler handler;
    private TextView txtBHBatteryStatus;
    private TextView txtBHConnectionStatus;
    private TextView txtBHBtName;
    private TextView bhConnectionStatusColor;
    // start-stop recording button
    private Button btnRecordStartStop;

    private TextView txtRecordingLocationInfo;
    private final Handler timerHandler = new Handler();
    private boolean locationInfoIsChecked = false;

    // TODO: 04.06.18 make configuration part of recording
    private Configuration configuration;
    private Recording recording;

    private BioHarness3 bH;
    private boolean isBhPaired;
    private boolean isBhPairing;

    /**
     * Service for location
     */
    private final ServiceConnection serviceConnection = new ServiceConnection() {
        @Override
        public void onServiceConnected(ComponentName name, IBinder service) {
            eDiaryLocationService.LocalBinder binder = (eDiaryLocationService.LocalBinder) service;
            binder.getService();
        }

        @Override
        public void onServiceDisconnected(ComponentName name) {
        }
    };


    /**
     * Method to acquire a timestamp of the current time without offset. (UTC)
     *
     * @return Calendar object representing current time in UTC.
     */
    public static Calendar getTimeStamp() {
        Calendar date = Calendar.getInstance();
        long currentTime_ms = date.getTimeInMillis();
        TimeZone localTimeZone = TimeZone.getDefault();
        int differenceLocalUTC = localTimeZone.getOffset(currentTime_ms);
        date.setTimeInMillis(currentTime_ms - differenceLocalUTC);
        return date;
    }

    /**
     * Check for Bluetooth.
     * taken from https://stackoverflow.com/a/21010026
     *
     * @return True if Bluetooth is available.
     */
    private static boolean isBluetoothAvailable() {
        final BluetoothAdapter bluetoothAdapter = BluetoothAdapter.getDefaultAdapter();
        return (bluetoothAdapter != null && bluetoothAdapter.isEnabled());
    }

    /**
     * Builds a Dialog to close the application and gives reason
     */
    private void alertCloseApplication(String title, String message) {

        Builder d = new AlertDialog.Builder(this)
                .setTitle(title).setMessage(message);

        d.setNegativeButton(getResources().getString(R.string.close_app),
                new DialogInterface.OnClickListener() {
                    public void onClick(DialogInterface dialog, int id) {
                        closeApp();
                    }
                });
        d.show();
    }

    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);


        // Filters for detecting Bluetooth connection status
        this.registerReceiver(BTReceiver, new IntentFilter(BluetoothDevice.ACTION_ACL_CONNECTED));
        this.registerReceiver(BTReceiver, new IntentFilter(BluetoothDevice.ACTION_ACL_DISCONNECTED));
        this.registerReceiver(BTReceiver, new IntentFilter(BluetoothDevice.ACTION_FOUND));
        this.registerReceiver(BTReceiver, new IntentFilter(BluetoothDevice.ACTION_BOND_STATE_CHANGED));
        this.registerReceiver(BTReceiver, new IntentFilter(BluetoothAdapter.ACTION_DISCOVERY_FINISHED));

        // Jakob Miksch: I tried to remove or replace this with getApplicationContext()
        //               but broke some AlertBuilder instances
        context = this;

        if (!isOnline()) {

            alertCloseApplication(getResources().getString(R.string.title_internet_problem),
                    getResources().getString(R.string.activate_internet)
            );

        } else if (hasDeniedPermissions(permissions)) {
            // on success start application, on failure close application
            requestPermissions(permissions);

        } else {
            // all permissions granted, start app functionality
            initialize();
        }

    }

    /**
     * Convenience method for activating the WakeLock
     */
    private void activateWakeLock() {
        PowerManager powerManager = (PowerManager) getSystemService(POWER_SERVICE);
        wakeLock = powerManager.newWakeLock(PowerManager.PARTIAL_WAKE_LOCK,
                "myapp:MyWakelockTag");
        wakeLock.acquire();
    }

    /**
     * Checks if device has access to internet
     * source: https://stackoverflow.com/a/4009133
     */
    private boolean isOnline() {
        ConnectivityManager cm =
                (ConnectivityManager) getSystemService(Context.CONNECTIVITY_SERVICE);
        NetworkInfo netInfo = cm.getActiveNetworkInfo();
        return netInfo != null && netInfo.isConnectedOrConnecting();
    }


    /**
     * Create menu of main screen (usually top right corner)
     */
    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        getMenuInflater().inflate(R.menu.menu, menu);
        return true;
    }

    /**
     * Start activity of selected menu item
     */
    @Override
    public boolean onOptionsItemSelected(MenuItem item) {

        int id = item.getItemId();

        if (id == R.id.about) {
            startActivity(new Intent(this, AboutActivity.class));
        }

        if (id == R.id.visualization) {
            startActivity(new Intent(this, RealTimeVisualization.class));
        }
        // check if menu option for location info is checked
        if (id == R.id.location_info_checkable_menu_item) {

            if (item.getItemId() == R.id.location_info_checkable_menu_item) {
                locationInfoIsChecked = !item.isChecked();
                item.setChecked(locationInfoIsChecked);
                return true;
            }
            return false;
        }
        return super.onOptionsItemSelected(item);
    }

    @Override
    public boolean onPrepareOptionsMenu(Menu menu) {
        invalidateOptionsMenu();

        // TODO: 05.10.18 this might be a duplicate call  
        // read configuaration if it has not happened yet
        configuration = readConfiguration();

        // hide visualization
        if (!configuration.isRealTimeVisualizationActivated()) {
            menu.findItem(R.id.visualization).setVisible(false);
        }

        MenuItem checkable = menu.findItem(R.id.location_info_checkable_menu_item);
        checkable.setChecked(locationInfoIsChecked);

        return super.onPrepareOptionsMenu(menu);
    }


    @Override
    protected void onDestroy() {
        wakeLock.release();
        unregisterReceiver(BTReceiver);
        super.onDestroy();
    }

    /**
     * Method for listening for the BT dialog response
     */
    protected void onActivityResult(int requestCode, int resultCode,
                                    Intent data) {
        if (requestCode == REQUEST_ENABLE_BT) {
            if (resultCode == RESULT_OK) {
                String e4APIKey = getAPIKeyOrCloseApp();
                sensorManager.handleConnect(e4APIKey);
            }
            if (resultCode == RESULT_CANCELED) {
                alertCloseApplication(getResources().getString(R.string.title_bluetooth_problem),
                        getResources().getString(R.string.activate_bluetooth));
            }
        }
    }

    /**
     * Allow the user to enable location in settings
     */
    private void showLocationDialog() {
        AlertDialog.Builder alertDialog = new AlertDialog.Builder(this);

        alertDialog.setCancelable(false);
        alertDialog.setTitle(getResources().getString(R.string.location_not_enabled));

        alertDialog.setMessage(getResources().getString(R.string.question_enable_location));

        alertDialog.setPositiveButton(getResources().getString(R.string.yes), new DialogInterface.OnClickListener() {
            public void onClick(DialogInterface dialog, int which) {
                Intent intent = new Intent(Settings.ACTION_LOCATION_SOURCE_SETTINGS);
                startActivity(intent);
            }
        });

        alertDialog.setNegativeButton(getResources().getString(R.string.close_app), new DialogInterface.OnClickListener() {
            public void onClick(DialogInterface dialog, int which) {
                closeApp();
            }
        });

        alertDialog.show();
    }


    // TODO: 22.03.2018 refactor and adapt to many sensors

    /**
     * Starts the functionality of the app, called when all permissions are granted
     */
    private void initialize() {
        // Check if location is disabled, show dialog where the user can enable it
        if (!isGPSEnabled()) {
            showLocationDialog();
        }

        activateWakeLock();

        Utils.createAppDirectory();

        Utils.makeUEconfigXml();

        configuration = readConfiguration();

        // check if Bluetooth is disabled, display a dialog to enable it
        if (!isBluetoothAvailable()) {
            Intent enableBtIntent = new Intent(BluetoothAdapter.ACTION_REQUEST_ENABLE);
            startActivityForResult(enableBtIntent, REQUEST_ENABLE_BT);
        }

        if (isOnline() && isBluetoothAvailable()) {
            String e4APIKey = getAPIKeyOrCloseApp();
            sensorManager.handleConnect(e4APIKey);
        }

        // create layout
        setContentView(R.layout.main);
        initRecordingStartStopButton();
        initSurveyButton();
        initMarkerButton();

        txtRecordingLocationInfo = findViewById(R.id.txtLocationInfo);
        // Sensor Status Info
        txtE4ConnectionStatus = findViewById(R.id.txt_e4_connection_status);
        txtE4BatteryStatus = findViewById(R.id.txt_e4_battery_status);

        txtBHBatteryStatus = findViewById(R.id.bh_battery);
        txtBHConnectionStatus = findViewById(R.id.bh_status);
        updateDeviceStatusTimer();

        initReconnectButton();

        bH = new BioHarness3();
        bhConnectionStatusColor = findViewById(R.id.txt_status_color_bh);
        txtBHBtName = findViewById(R.id.txt_bh_bt_name);
    }

    @Override
    public void onPause() {
        // pause handler/thread for recording information when leaving activity
        timerHandler.removeCallbacksAndMessages(timerRunnable);
        super.onPause();
    }

    @Override
    public void onResume() {
        // resume handler/thread for recording information when resuming activity
        timerHandler.postDelayed(timerRunnable, 200);

        super.onResume();
    }

    private void initSurveyButton() {
        Button btnSurvey = findViewById(R.id.btn_survey);

        if (configuration.isSurveyActivated()) {
            btnSurvey.setOnClickListener(new View.OnClickListener() {
                @Override
                public void onClick(View view) {

                    // start recording if it has not started yet
                    if (recording == null || recording.isNotRunning()) {
                        startRecording();
                    }

                    Intent surveyIntent = new Intent(eDiary.this, SurveyActivity.class);
                    startActivity(surveyIntent);
                }
            });
        } else {
            btnSurvey.setVisibility(View.INVISIBLE);
        }
    }

    private void initMarkerButton() {
        Button btnMarker = findViewById(R.id.btn_marker);

        if (configuration.isMarkerButtonActivated()) {
            btnMarker.setOnClickListener(new View.OnClickListener() {
                @Override
                public void onClick(View view) {
                    // start recording if it has not started yet
                    if (recording == null || recording.isNotRunning()) {
                        startRecording();
                    }
                    if (recording.getLastLocation() != null) {

                        Marker marker = new Marker(getTimeStamp(), recording.getLastLocation());
                        recording.submitMarkerToDB(marker);

                        Toast.makeText(getApplicationContext(), getResources().getString(R.string.marker_submitted), Toast.LENGTH_LONG).show();
                    } else {

                        Toast.makeText(getApplicationContext(), getResources().getString(R.string.location_unavailable), Toast.LENGTH_LONG).show();

                    }
                }
            });

        } else {
            btnMarker.setVisibility(View.INVISIBLE);
        }
    }

    private void initReconnectButton() {
        // Reconnect Button
        Button btnReconnect = findViewById(R.id.btn_reconnect);
        // restart app
        btnReconnect.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {

                new Builder(eDiary.context)
                        .setTitle(getResources().getString(R.string.reconnecting_sensor))
                        .setMessage(getResources().getString(R.string.question_reconnect))
                        .setNegativeButton(getResources().getString(R.string.no_do_no_reconnect), new DialogInterface.OnClickListener() {
                            @Override
                            public void onClick(DialogInterface dialog, int which) {
                                dialog.cancel();
                            }
                        })
                        .setPositiveButton(getResources().getString(R.string.yes_quit_session_and_reconnect), new DialogInterface.OnClickListener() {
                            @Override
                            public void onClick(DialogInterface dialog, int which) {

                                // Restart application
                                // source: https://stackoverflow.com/a/17166729
                                Intent mStartActivity = new Intent(eDiary.this, eDiary.class);
                                int mPendingIntentId = 123456;
                                PendingIntent mPendingIntent = PendingIntent.getActivity(eDiary.this, mPendingIntentId, mStartActivity,
                                        PendingIntent.FLAG_CANCEL_CURRENT);
                                AlarmManager mgr = (AlarmManager) eDiary.this.getSystemService(Context.ALARM_SERVICE);
                                mgr.set(AlarmManager.RTC, System.currentTimeMillis() + 100, mPendingIntent);
                                System.exit(0);
                            }
                        })
                        .create().show();
            }
        });
    }

    private void initRecordingStartStopButton() {
        // Button "start/stop Recording"
        btnRecordStartStop = findViewById(R.id.btn_record_start_stop);
        btnRecordStartStop.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                if (recording == null || recording.isNotRunning()) {
                    startRecording();

                } else {
                    // Dialog if user wants to finish Session
                    new Builder(eDiary.context)
                            .setTitle(getResources().getString(R.string.title_stop_recording))
                            .setMessage(getResources().getString(R.string.question_stop_recording))
                            .setNegativeButton(getResources().getString(R.string.no_continue_recording), new DialogInterface.OnClickListener() {
                                @Override
                                public void onClick(DialogInterface dialog, int which) {
                                    dialog.cancel();
                                }
                            })
                            .setPositiveButton(getResources().getString(R.string.yes_stop_recording), new DialogInterface.OnClickListener() {
                                @Override
                                public void onClick(DialogInterface dialog, int which) {

                                    Intent intent = new Intent(eDiary.this, eDiaryLocationService.class);
                                    stopService(intent);

                                    //finish session and close app
                                    recording.stopAndFlush();
                                    closeApp();
                                }
                            })
                            .create().show();
                }
            }
        });
    }

    private Configuration readConfiguration() {
        configuration = new Configuration();
        try {
            configuration.importConfiguration();
        } catch (Exception e) {
            // TODO: 07.05.18 catch different types of exceptions
            e.printStackTrace();
        }
        return configuration;
    }

    /**
     * Reads Empatica E4 API key from config
     * if not successful, dialog pops up and app closes
     */
    private String getAPIKeyOrCloseApp() {

        // Try to find API key
        List<HashMap<String, String>> sensors = configuration.getSensors();
        String e4APIKey = "";
        for (int i = 0; i < sensors.size(); i++) {
            HashMap<String, String> sensor = sensors.get(i);

            String platformName = sensor.get(Configuration.PLATFORM);
            String e4Name = Constants.SensorPlatform.EMPATICA_E4.toString();

            if (Objects.equals(platformName, e4Name)) {
                e4APIKey = sensor.get(Configuration.API_KEY);
                break;
            }
        }

        Log.d(TAG, "e4APIKey: " + e4APIKey);

        // no API key is found
        if (Objects.equals(e4APIKey, "")) {
            new Builder(eDiary.context)
                    .setTitle(getResources().getString(R.string.alert_api_key_not_found))
                    .setMessage(getResources().getString(R.string.e4_api_key_not_found_please_add_to_config_file))
                    .setNeutralButton(getResources().getString(R.string.close_app), new DialogInterface.OnClickListener() {
                        @Override
                        public void onClick(DialogInterface dialog, int which) {
                            closeApp();
                        }
                    })
                    .create().show();
        }
        return e4APIKey;
    }

    /**
     * Uses the recording location information (same one that is submitted to the DB) as information about the
     * last recording, time and date is shown in Central European Standard Time along with location provider
     * and accuracy. These location parameters can be adjusted to display more or less information to the user
     */
    private void setRecordingLocationStatus() {

        //Recording location info
        Location lastRecordingLocation = recording.getLastLocation();

        if (lastRecordingLocation != null) {

            String locationProvider = lastRecordingLocation.getProvider();
            long locationTime = lastRecordingLocation.getTime();
            float locationAccuracy = lastRecordingLocation.getAccuracy();

            Date locTimeReal = new Date(locationTime);
            DateFormat formatter = new SimpleDateFormat();
            formatter.getTimeZone();
            String locationTimeFormatted = formatter.format(locTimeReal);

            // maybe better to split this up into separate TextViews
            String lastRecordingInfo = (getResources().getString(R.string.last_recording_time_info) + " " + locationTimeFormatted +
                    "\n" + getResources().getString(R.string.last_recording_accu_info) + " " + locationAccuracy + " m" +
                    "\n" + getResources().getString(R.string.last_recording_provider_info) + " " + locationProvider.toUpperCase());

            txtRecordingLocationInfo.setText(lastRecordingInfo);
        } else {
            txtRecordingLocationInfo.setText(getResources().getString(R.string.location_unavailable));
        }

    }

    /**
     * Runnable for setting the location info, the info is displayed when recording is started
     */
    private final Runnable timerRunnable = new Runnable() {
        @Override
        public void run() {
            // if recording is running and the option to see recording location info is checked, display the info
            if (recording != null && !recording.isNotRunning() && locationInfoIsChecked) {
                txtRecordingLocationInfo.setVisibility(View.VISIBLE);
                setRecordingLocationStatus();

            }
            // if location is unchecked in the menu stop displaying it
            else if (!locationInfoIsChecked && txtRecordingLocationInfo != null) {

                timerHandler.removeCallbacksAndMessages(timerRunnable);
                txtRecordingLocationInfo.setVisibility(View.INVISIBLE);
            }
            // run again after 200 milliseconds (1/5 sec)
            timerHandler.postDelayed(this, 200);
        }
    };

    // TODO: 12.03.2018 documentation and refactoring ( so many if/else try/catch :-O )

    /**
     * Executes necessary commands for starting a session
     * only called by "start recording" button and "submit" button of survey
     */
    private void startRecording() {

        Intent intent = new Intent(eDiary.this, eDiaryLocationService.class);
        bindService(intent, serviceConnection, Context.BIND_AUTO_CREATE);
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.O) {
            startForegroundService(intent);
        } else {
            startService(intent);
        }

        recording = Recording.getInstance();

        Survey survey = configuration.getSurvey();
        recording.start(survey);

        btnRecordStartStop.setText(R.string.stop);

        // User Info Recording
        TextView txtUserInfo = findViewById(R.id.txt_user_info);
        txtUserInfo.setText(R.string.recording_running);

        Toast.makeText(getApplicationContext(), getResources().getString(R.string.recording_has_started), Toast.LENGTH_LONG).show();

    }

    /**
     * Checks if GPS or Network location is enabled
     */
    private boolean isGPSEnabled() {

        LocationManager locationService = (LocationManager) getSystemService(LOCATION_SERVICE);

        boolean GPSEnabled = false;

        // prevent null pointer exception
        if (locationService != null) {
            GPSEnabled = locationService.isProviderEnabled(LocationManager.GPS_PROVIDER);
        }

        return GPSEnabled;
    }

    /**
     * Requests multiple permissions
     */
    private void requestPermissions(String[] permissions) {

        String[] deniedPermissions = filterDeniedPermissions(permissions);

        // no denied permissions - everything is permitted
        if (deniedPermissions.length == 0) {
            return;
        }

        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.M) {
            // invokes callback method "onRequestPermissionsResult"
            requestPermissions(deniedPermissions, 0);
        }
    }

    private boolean hasDeniedPermissions(String[] permissions) {
        for (String permission : permissions) {
            if (ContextCompat.checkSelfPermission(eDiary.this, permission) != PackageManager.PERMISSION_GRANTED) {
                return true;
            }
        }
        return false;
    }

    /**
     * Returns permissions that are denied (not permitted)
     */
    private String[] filterDeniedPermissions(String[] permissions) {
        List<String> filteredPermissions = new ArrayList<>();
        for (String permission : permissions) {
            if (ContextCompat.checkSelfPermission(eDiary.this, permission) != PackageManager.PERMISSION_GRANTED) {
                filteredPermissions.add(permission);
            }
        }
        return filteredPermissions.toArray(new String[0]);
    }

    /**
     * Callback method that is executed after requestPermissions
     * checks if permissions have been denied many times and shows info to the user
     * if everything is fine, app will be launched with initialize()
     */
    @Override
    public void onRequestPermissionsResult(final int requestCode, final String[] permissions,
                                           int[] grantResults) {
        Log.d(TAG, "onRequestPermissionsResult() invoked: " + Arrays.toString(permissions));

        // after second times asking an Alert will be prompted
        if (requestCode >= 1) {
            // TODO: 15.03.2018 not sure if requestCode should be used like this
            // TODO: in the example by Google they use constant numbers
            // TODO: see https://developer.android.com/training/permissions/requesting.html
            Log.d(TAG, "Too many permission requests, displaying App-Settings dialog.");
            new AlertDialog.Builder(eDiary.context)
                    .setTitle("Permissions required!")
                    .setMessage("This app can't run without granting all the required permissions. " +
                            System.lineSeparator() + System.lineSeparator() +
                            "You can set the permissions manually via Androids application settings, " +
                            "if you accidentally disabled the display of a permissions dialog.")
                    .setPositiveButton("Open Settings", new DialogInterface.OnClickListener() {
                        @Override
                        public void onClick(DialogInterface dialog, int which) {
                            try {
                                //Open the specific App Info page:
                                Intent intent = new Intent(android.provider.Settings.ACTION_APPLICATION_DETAILS_SETTINGS);
                                intent.setData(Uri.parse("package:" + eDiary.context.getPackageName()));
                                startActivity(intent);

                            } catch (ActivityNotFoundException e) {
                                //e.printStackTrace();
                                Log.e(TAG, "Alert Dialog", e);

                                //Open the generic Apps page:
                                Intent intent = new Intent(android.provider.Settings.ACTION_MANAGE_APPLICATIONS_SETTINGS);
                                startActivity(intent);
                            }
                            closeApp();
                        }
                    })
                    .setNeutralButton("Cancel", new DialogInterface.OnClickListener() {
                        @Override
                        public void onClick(DialogInterface dialog, int which) {

                            closeApp();
                        }
                    })
                    .create().show();
            return;
        }

        final ArrayList<String> deniedPermissions = new ArrayList<>();

        for (int i = 0; i < permissions.length; i++) {
            if (grantResults[i] != PackageManager.PERMISSION_GRANTED) {
                deniedPermissions.add(permissions[i]);
            }
        }

        // some permissions are not granted
        if (!deniedPermissions.isEmpty()) {
            android.util.Log.d(TAG, "Not all permissions where granted.");
            ActivityCompat.requestPermissions(eDiary.this, deniedPermissions.toArray(new String[0]), requestCode + 1);

            // all permissions granted
        } else {
            initialize();
        }
    }

    private void closeApp() {
        Process.killProcess(Process.myPid());
    }


    /**
     * Listens to changes in the connection status with the BioHarness as well as
     * discovery and bonding status. Taken from https://developer.android.com/guide/topics/connectivity/bluetooth
     */

    private final BroadcastReceiver BTReceiver = new BroadcastReceiver() {

        @Override
        public void onReceive(Context context, Intent intent) {
            String action = intent.getAction();


            // ACTION_ACL_CONNECTED intentionally left empty
            if (BluetoothDevice.ACTION_ACL_DISCONNECTED.equals(action)) {
                bH.onDisconnect();
            } else if (BluetoothDevice.ACTION_FOUND.equals(action)) {
                // Called when any device is discovered, check if discovered device is BioHarness, if true initiate pairing process

                BluetoothDevice device = intent.getParcelableExtra(BluetoothDevice.EXTRA_DEVICE);
                String deviceName = device.getName();
                String deviceHardwareAddress = device.getAddress();

                if (deviceName != null && deviceName.toLowerCase().startsWith("bh")) {
                    isBhPairing = true;
                    bH.btAdapter.cancelDiscovery();
                    Log.i(TAG, "Bluetooth device discovery canceled");
                    device.createBond();
                    Log.i(TAG, "Pairing process with " + deviceName + " started");
                }
                Log.i(TAG, "Device discovery found " + deviceName + "Mac: " + deviceHardwareAddress);
            } else if (BluetoothDevice.ACTION_BOND_STATE_CHANGED.equals(action)) {
                //Called when bluetooth device bond state changes
                BluetoothDevice device = intent.getParcelableExtra(BluetoothDevice.EXTRA_DEVICE);
                int pairingStatus = device.getBondState();
                String deviceName = device.getName();
                // if device is bonded/paired try to connect to it
                if (pairingStatus == 12) {
                    Log.i(TAG, "BioHarness paired, attempting to connect.");
                    isBhPaired = true;
                    bH.connectToBh();

                } else if (pairingStatus == 10) {
                    isBhPaired = false;
                    Toast.makeText(getApplicationContext(), "Pairing with " + deviceName + " failed", Toast.LENGTH_LONG).show();
                    bH.onDisconnect();
                }

            } else if (BluetoothAdapter.ACTION_DISCOVERY_FINISHED.equals(action)) {
                Log.i(TAG, "Bluetooth device discovery finished");
                if (!isBhPaired && !isBhPairing) {
                    discoveryFailDialog();
                }
            }
        }
    };

    /**
     * Creates dialog for when Bluetooth fails to find BioHarness
     */
    private void discoveryFailDialog() {
        AlertDialog.Builder alertDialog = new AlertDialog.Builder(this);

        alertDialog.setCancelable(true);
        alertDialog.setTitle(getResources().getString(R.string.discovery_fail_title));

        alertDialog.setMessage(getResources().getString(R.string.discovery_fail_msg));

        alertDialog.setPositiveButton(getResources().getString(R.string.bluetooth_settings), new DialogInterface.OnClickListener() {
            public void onClick(DialogInterface dialog, int which) {
                Intent intent = new Intent(Settings.ACTION_BLUETOOTH_SETTINGS);
                startActivity(intent);
            }
        });

        alertDialog.setNegativeButton(getResources().getString(R.string.cancel), new DialogInterface.OnClickListener() {
            public void onClick(DialogInterface dialog, int which) {
                dialog.cancel();
            }
        });

        alertDialog.show();
    }


    /**
     * Checks regularly the status of E4 and BioHarness, and displays it on the main screen
     * Activates Record button after connection has been established
     */
    private void updateDeviceStatusTimer() {

        handler = new Handler();
        Runnable runnable = new Runnable() {
            @Override
            public void run() {

                // get sensor information from SensorStatus class
                SensorStatusEntry e4Details = SensorStatusCollection.getDetails().get(Constants.SensorPlatform.EMPATICA_E4);
                SensorStatusEntry bhDetails = SensorStatusCollection.getDetails().get(Constants.SensorPlatform.BIOHARNESS_3);

                if (bhDetails != null) {
                    String bhBattery = bhDetails.getBatteryStatus();
                    SensorPlatform.ConnectionStatus bhConnectionStatus = bhDetails.getConnectionStatus();

                    txtBHBatteryStatus.setText(bhBattery + " %");
                    txtBHConnectionStatus.setText(bhConnectionStatus.toString());

                    Log.v(TAG, "BH Status: " + bhConnectionStatus + " - " + bhBattery + " %");

                    if (bH.getPairedDeviceName() != null) {
                        txtBHBtName.setText(bH.getPairedDeviceName());
                    }

                    switch (bhConnectionStatus) {
                        case CONNECTED:
                            bhConnectionStatusColor.setBackgroundColor(ContextCompat.getColor(context, R.color.green));
                            break;
                        case CONNECTING:
                            bhConnectionStatusColor.setBackgroundColor(ContextCompat.getColor(context, R.color.orange));
                            break;
                        default:
                            bhConnectionStatusColor.setBackgroundColor(ContextCompat.getColor(context, R.color.red));
                            break;
                    }

                }

                // check if it already contains values
                if (e4Details != null) {

                    // extract sensor information
                    String e4CBatteryStatus = e4Details.getBatteryStatus();
                    SensorPlatform.ConnectionStatus e4ConnectionStatus = e4Details.getConnectionStatus();

                    // set sensor info in text views
                    txtE4ConnectionStatus.setText(e4ConnectionStatus.toString());
                    txtE4BatteryStatus.setText(e4CBatteryStatus + " %");
                    Log.v(TAG, "E4 Status: " + txtE4ConnectionStatus + " - " + txtE4BatteryStatus + " %");

                    // status color for connection
                    TextView txtE4StatusColor = findViewById(R.id.txt_status_color);
                    switch (e4ConnectionStatus) {
                        case CONNECTED:
                            txtE4StatusColor.setBackgroundColor(ContextCompat.getColor(context, R.color.green));
                            break;
                        case CONNECTING:
                            txtE4StatusColor.setBackgroundColor(ContextCompat.getColor(context, R.color.orange));
                            break;
                        default:
                            txtE4StatusColor.setBackgroundColor(ContextCompat.getColor(context, R.color.red));
                            break;
                    }

                }

                handler.postDelayed(this, 1000);
            }
        };
        // TODO: 12.03.2018 why handler two times ?
        handler.postDelayed(runnable, 1000);
    }

}
