package at.ac.sbg.zgis.util;

import android.os.Environment;
import android.util.Log;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;

import at.ac.sbg.zgis.configuration.Configuration;
import at.ac.sbg.zgis.configuration.Constants;
import at.ac.sbg.zgis.userinterface.eDiary;

/**
 * Contains utility functions.
 */
public class Utils {
    private static final String TAG = Utils.class.getSimpleName();

    /**
     * Checks if a config file exist,
     * if not it creates a new one
     */
    public static void makeUEconfigXml() {
        File appDirectory = new File(Environment.getExternalStorageDirectory(), Constants.extDir);
        File toFile = new File(appDirectory, Configuration.CONFIG_FILE_NAME);
        if (toFile.exists()) {
            Log.d(TAG, "Found config.xml on the device.");
            return;
        }
        Log.d(TAG, "Found no config.xml on the device, creating from resources.");
        try {
            InputStream inStream = eDiary.context.getAssets().open(Configuration.CONFIG_FILE_NAME);
            BufferedReader br = new BufferedReader(new InputStreamReader(inStream));
            copyAssetFile(br, toFile);
            Log.d(TAG, "Config file creation succeeded.");
        } catch (IOException e) {
            Log.e(TAG, "Creating of config file failed.", e);
        }
    }

    /**
     * Convenience method for copying files
     */
    private static void copyAssetFile(BufferedReader br, File toFile) throws IOException {
        BufferedWriter bw = null;
        try {
            bw = new BufferedWriter(new FileWriter(toFile));

            int in;
            while ((in = br.read()) != -1) {
                bw.write(in);
            }
        } finally {
            if (bw != null) {
                bw.close();
            }
            br.close();
        }
    }

    /**
     * Creates a directory for the application in the root directory of the phone
     * does nothing, if this directory already exists
     */
    public static void createAppDirectory() {
        File sdCard = Environment.getExternalStorageDirectory();
        File dataDirectory = new File(sdCard.getAbsolutePath() + "/" + Constants.extDir);

        if (!dataDirectory.exists()) {
            if (dataDirectory.mkdirs()) {
                Log.d(TAG, "App directory created.");
            } else {
                Log.d(TAG, "Creation of app directory failed.");
            }
        }else {
            Log.d(TAG, "No directory created. App directory already exist.");
        }
    }

    public static File getOrCreateExternalDirectory(String dirName) {
        String state = Environment.getExternalStorageState();
        File directory = null;
        if (Environment.MEDIA_MOUNTED.equals(state)) {
            File ueExtDir = new File(Environment.getExternalStorageDirectory(), Constants.extDir);
            directory = new File(ueExtDir, dirName);
            directory.mkdirs();
        }
        if (!directory.isDirectory()) {
            return null;
        }
        return directory;
    }
}
