#!/bin/bash

# This file iterates through all connected phones
# and pulls the eDiary data and processes it

# check argument
if [ -z "$1" ];then
  target_dir=`pwd`
else
  target_dir=$1
fi

# copy eDiary data from phone to computer and
# do some processing
# first argument: phone_id
# second argument: target_dir
copy_and_process_ediary_data() {

  phone_id=$1
  # get phone_name with an adb request
  phone_name=`adb -s $phone_id shell settings get global device_name`

  target_dir=$2
  phone_dir=$target_dir/$phone_name
  mkdir -p $phone_dir

  ## copy data from phone to computer
  adb -s $phone_id shell 'ls sdcard/eDiary/sqlite/*sqlite' |
  tr -d '\r' |
  xargs -n1 \
    -I file_name \
    adb -s $phone_id pull file_name $phone_dir

  ## convert sqlite files into GeoPackage
  ## TODO: replace this codeblock with an external function
  sqlite_suffix=.sqlite
  for file in $phone_dir/*$sqlite_suffix;
  do
    echo $file;
    filename_without_suffix=${file%$sqlite_suffix};
    echo $filename_without_suffix;
    ogr2ogr \
      -f "GPKG" $filename_without_suffix.gpkg \
      $file \
      -sql \
      "SELECT
       *,
       MakePoint(longitude, latitude, 4326) AS geometry
     FROM
    location" \
      -nln "location" \
      -a_srs "EPSG:4326";

    # create a diagram of the sqlite file
    ediary.py $file;

    # move data into a folder with timestamp
    # this separates the data from the new data
    # but does not delete it
    backup_ediary_on_phone.sh
  done

  # convert all GeoPackages into one
  # useful if the run consists of many sessions
  # e.g. when the E4 got disconnected
  for geopackage in */*.gpkg; do
    ogr2ogr \
      -f "GPKG" $phone_dir/${phone_name}_combined_tracks.gpkg \
      -append \
      $geopackage
  done

}

# this line is necessary in order that
# xargs can use this function
export -f copy_and_process_ediary_data;

# execute the upper function
adb devices |
awk 'NR>1{print $1}' |
xargs \
  -n1 \
  -I phone_id \
  bash -c 'copy_and_process_ediary_data phone_id '$target_dir
